# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["CreateBoxConfigParam"]


class CreateBoxConfigParam(TypedDict, total=False):
    envs: Dict[str, str]
    """Environment variables for the box.

    These variables will be available in all operations including command execution,
    code running, and other box behaviors
    """

    labels: Dict[str, str]
    """Key-value pairs of labels for the box.

    Labels are used to add custom metadata to help identify, categorize, and manage
    boxes. Common use cases include project names, environments, teams,
    applications, or any other organizational tags that help you organize and filter
    your boxes.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from ...._types import FileTypes

__all__ = ["AndroidInstallParams", "InstallAndroidPkgByFile", "InstallAndroidPkgByURL"]


class InstallAndroidPkgByFile(TypedDict, total=False):
    apk: Required[FileTypes]
    """APK file to install (max file size: 512MB)"""


class InstallAndroidPkgByURL(TypedDict, total=False):
    apk: Required[str]
    """HTTP URL to download APK file (max file size: 512MB)"""


AndroidInstallParams: TypeAlias = Union[InstallAndroidPkgByFile, InstallAndroidPkgByURL]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import browser_cdp_url_params

__all__ = ["BrowserResource", "AsyncBrowserResource"]


class BrowserResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BrowserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return BrowserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrowserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return BrowserResourceWithStreamingResponse(self)

    def cdp_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Generate pre-signed CDP url

        Args:
          expires_in: The CDP url will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 120m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/browser/connect-url/cdp",
            body=maybe_transform({"expires_in": expires_in}, browser_cdp_url_params.BrowserCdpURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncBrowserResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBrowserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncBrowserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrowserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncBrowserResourceWithStreamingResponse(self)

    async def cdp_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Generate pre-signed CDP url

        Args:
          expires_in: The CDP url will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 120m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/browser/connect-url/cdp",
            body=await async_maybe_transform({"expires_in": expires_in}, browser_cdp_url_params.BrowserCdpURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class BrowserResourceWithRawResponse:
    def __init__(self, browser: BrowserResource) -> None:
        self._browser = browser

        self.cdp_url = to_raw_response_wrapper(
            browser.cdp_url,
        )


class AsyncBrowserResourceWithRawResponse:
    def __init__(self, browser: AsyncBrowserResource) -> None:
        self._browser = browser

        self.cdp_url = async_to_raw_response_wrapper(
            browser.cdp_url,
        )


class BrowserResourceWithStreamingResponse:
    def __init__(self, browser: BrowserResource) -> None:
        self._browser = browser

        self.cdp_url = to_streamed_response_wrapper(
            browser.cdp_url,
        )


class AsyncBrowserResourceWithStreamingResponse:
    def __init__(self, browser: AsyncBrowserResource) -> None:
        self._browser = browser

        self.cdp_url = async_to_streamed_response_wrapper(
            browser.cdp_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .android_app import AndroidApp as AndroidApp
from .f_info_params import FInfoParams as FInfoParams
from .f_list_params import FListParams as FListParams
from .f_read_params import FReadParams as FReadParams
from .f_write_params import FWriteParams as FWriteParams
from .f_exists_params import FExistsParams as FExistsParams
from .f_info_response import FInfoResponse as FInfoResponse
from .f_list_response import FListResponse as FListResponse
from .f_read_response import FReadResponse as FReadResponse
from .f_remove_params import FRemoveParams as FRemoveParams
from .f_rename_params import FRenameParams as FRenameParams
from .f_write_response import FWriteResponse as FWriteResponse
from .f_exists_response import FExistsResponse as FExistsResponse
from .f_remove_response import FRemoveResponse as FRemoveResponse
from .f_rename_response import FRenameResponse as FRenameResponse
from .action_drag_params import ActionDragParams as ActionDragParams
from .action_move_params import ActionMoveParams as ActionMoveParams
from .action_type_params import ActionTypeParams as ActionTypeParams
from .action_click_params import ActionClickParams as ActionClickParams
from .action_swipe_params import ActionSwipeParams as ActionSwipeParams
from .action_touch_params import ActionTouchParams as ActionTouchParams
from .android_open_params import AndroidOpenParams as AndroidOpenParams
from .action_drag_response import ActionDragResponse as ActionDragResponse
from .action_move_response import ActionMoveResponse as ActionMoveResponse
from .action_scroll_params import ActionScrollParams as ActionScrollParams
from .action_type_response import ActionTypeResponse as ActionTypeResponse
from .android_get_response import AndroidGetResponse as AndroidGetResponse
from .action_click_response import ActionClickResponse as ActionClickResponse
from .action_swipe_response import ActionSwipeResponse as ActionSwipeResponse
from .action_touch_response import ActionTouchResponse as ActionTouchResponse
from .action_scroll_response import ActionScrollResponse as ActionScrollResponse
from .android_install_params import AndroidInstallParams as AndroidInstallParams
from .android_restart_params import AndroidRestartParams as AndroidRestartParams
from .android_restore_params import AndroidRestoreParams as AndroidRestoreParams
from .browser_cdp_url_params import BrowserCdpURLParams as BrowserCdpURLParams
from .action_press_key_params import ActionPressKeyParams as ActionPressKeyParams
from .android_list_pkg_params import AndroidListPkgParams as AndroidListPkgParams
from .action_screenshot_params import ActionScreenshotParams as ActionScreenshotParams
from .android_install_response import AndroidInstallResponse as AndroidInstallResponse
from .android_uninstall_params import AndroidUninstallParams as AndroidUninstallParams
from .browser_cdp_url_response import BrowserCdpURLResponse as BrowserCdpURLResponse
from .action_press_key_response import ActionPressKeyResponse as ActionPressKeyResponse
from .android_list_app_response import AndroidListAppResponse as AndroidListAppResponse
from .android_list_pkg_response import AndroidListPkgResponse as AndroidListPkgResponse
from .action_press_button_params import ActionPressButtonParams as ActionPressButtonParams
from .action_screenshot_response import ActionScreenshotResponse as ActionScreenshotResponse
from .action_press_button_response import ActionPressButtonResponse as ActionPressButtonResponse
from .action_screen_rotation_params import ActionScreenRotationParams as ActionScreenRotationParams
from .android_list_pkg_simple_params import AndroidListPkgSimpleParams as AndroidListPkgSimpleParams
from .action_screen_rotation_response import ActionScreenRotationResponse as ActionScreenRotationResponse
from .android_list_activities_response import AndroidListActivitiesResponse as AndroidListActivitiesResponse
from .android_list_pkg_simple_response import AndroidListPkgSimpleResponse as AndroidListPkgSimpleResponse
from .android_get_connect_address_response import AndroidGetConnectAddressResponse as AndroidGetConnectAddressResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["FReadResponse"]


class FReadResponse(BaseModel):
    content: str
    """Content of the file"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Union, cast
from typing_extensions import Literal

import httpx

from .fs import (
    FsResource,
    AsyncFsResource,
    FsResourceWithRawResponse,
    AsyncFsResourceWithRawResponse,
    FsResourceWithStreamingResponse,
    AsyncFsResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .android import (
    AndroidResource,
    AsyncAndroidResource,
    AndroidResourceWithRawResponse,
    AsyncAndroidResourceWithRawResponse,
    AndroidResourceWithStreamingResponse,
    AsyncAndroidResourceWithStreamingResponse,
)
from .browser import (
    BrowserResource,
    AsyncBrowserResource,
    BrowserResourceWithRawResponse,
    AsyncBrowserResourceWithRawResponse,
    BrowserResourceWithStreamingResponse,
    AsyncBrowserResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import (
    box_list_params,
    box_stop_params,
    box_start_params,
    box_run_code_params,
    box_terminate_params,
    box_create_linux_params,
    box_live_view_url_params,
    box_create_android_params,
    box_execute_commands_params,
    box_web_terminal_url_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.linux_box import LinuxBox
from ....types.v1.android_box import AndroidBox
from ....types.v1.box_list_response import BoxListResponse
from ....types.v1.box_stop_response import BoxStopResponse
from ....types.v1.box_start_response import BoxStartResponse
from ....types.v1.box_retrieve_response import BoxRetrieveResponse
from ....types.v1.box_run_code_response import BoxRunCodeResponse
from ....types.v1.create_box_config_param import CreateBoxConfigParam
from ....types.v1.box_live_view_url_response import BoxLiveViewURLResponse
from ....types.v1.box_execute_commands_response import BoxExecuteCommandsResponse
from ....types.v1.box_web_terminal_url_response import BoxWebTerminalURLResponse

__all__ = ["BoxesResource", "AsyncBoxesResource"]


class BoxesResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def fs(self) -> FsResource:
        return FsResource(self._client)

    @cached_property
    def browser(self) -> BrowserResource:
        return BrowserResource(self._client)

    @cached_property
    def android(self) -> AndroidResource:
        return AndroidResource(self._client)

    @cached_property
    def with_raw_response(self) -> BoxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return BoxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BoxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return BoxesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxRetrieveResponse:
        """
        Get box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxRetrieveResponse,
            self._get(
                f"/boxes/{box_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, BoxRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        device_type: str | NotGiven = NOT_GIVEN,
        labels: object | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: List[Literal["all", "pending", "running", "error", "terminated"]] | NotGiven = NOT_GIVEN,
        type: List[Literal["all", "linux", "android"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxListResponse:
        """
        List box

        Args:
          device_type: Filter boxes by their device type (virtual, physical)

          labels: Filter boxes by their labels. Labels are key-value pairs that help identify and
              categorize boxes. Use this to filter boxes that match specific label criteria.
              For example, you can filter by project, environment, team, or any custom labels
              you've added to your boxes.

          page: Page number

          page_size: Page size

          status: Filter boxes by their current status (pending, running, stopped, error,
              terminated, all). Must be an array of statuses. Use 'all' to get boxes with any
              status.

          type: Filter boxes by their type (linux, android, all). Must be an array of types. Use
              'all' to get boxes of any type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/boxes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "device_type": device_type,
                        "labels": labels,
                        "page": page,
                        "page_size": page_size,
                        "status": status,
                        "type": type,
                    },
                    box_list_params.BoxListParams,
                ),
            ),
            cast_to=BoxListResponse,
        )

    def create_android(
        self,
        *,
        config: box_create_android_params.Config | NotGiven = NOT_GIVEN,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidBox:
        """
        Create android box

        Args:
          config: Configuration for a Android box instance

          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/boxes/android",
            body=maybe_transform(
                {
                    "config": config,
                    "wait": wait,
                },
                box_create_android_params.BoxCreateAndroidParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidBox,
        )

    def create_linux(
        self,
        *,
        config: CreateBoxConfigParam | NotGiven = NOT_GIVEN,
        expires_in: str | NotGiven = NOT_GIVEN,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LinuxBox:
        """
        Create linux box

        Args:
          config: Configuration for a box instance

          expires_in: The box will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 60m

          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/boxes/linux",
            body=maybe_transform(
                {
                    "config": config,
                    "expires_in": expires_in,
                    "wait": wait,
                },
                box_create_linux_params.BoxCreateLinuxParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LinuxBox,
        )

    def execute_commands(
        self,
        box_id: str,
        *,
        commands: Union[str, List[str]],
        envs: object | NotGiven = NOT_GIVEN,
        api_timeout: str | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxExecuteCommandsResponse:
        """Exec command

        Args:
          commands: The command to run.

        Can be a single string or an array of strings

          envs: The environment variables to run the command

          api_timeout: The timeout of the command. If the command times out, the exit code will be 124.
              For example: 'timeout 5s sleep 10s' will result in exit code 124.

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30s

          working_dir: The working directory of the command. It not provided, the command will be run
              in the `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/commands",
            body=maybe_transform(
                {
                    "commands": commands,
                    "envs": envs,
                    "api_timeout": api_timeout,
                    "working_dir": working_dir,
                },
                box_execute_commands_params.BoxExecuteCommandsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxExecuteCommandsResponse,
        )

    def live_view_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxLiveViewURLResponse:
        """
        Generate pre-signed live view url

        Args:
          expires_in: The live view will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 180m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/live-view-url",
            body=maybe_transform({"expires_in": expires_in}, box_live_view_url_params.BoxLiveViewURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxLiveViewURLResponse,
        )

    def run_code(
        self,
        box_id: str,
        *,
        code: str,
        argv: List[str] | NotGiven = NOT_GIVEN,
        envs: object | NotGiven = NOT_GIVEN,
        language: Literal["bash", "python", "typescript"] | NotGiven = NOT_GIVEN,
        api_timeout: str | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxRunCodeResponse:
        """
        Run code on the box

        Args:
          code: The code to run

          argv: The arguments to run the code. For example, if you want to run "python index.py
              --help", you should pass ["--help"] as arguments.

          envs: The environment variables to run the code

          language: The language of the code.

          api_timeout: The timeout of the code execution. If the code execution times out, the exit
              code will be 124.

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30s

          working_dir: The working directory of the code. It not provided, the code will be run in the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/run-code",
            body=maybe_transform(
                {
                    "code": code,
                    "argv": argv,
                    "envs": envs,
                    "language": language,
                    "api_timeout": api_timeout,
                    "working_dir": working_dir,
                },
                box_run_code_params.BoxRunCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxRunCodeResponse,
        )

    def start(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxStartResponse:
        """
        Start box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxStartResponse,
            self._post(
                f"/boxes/{box_id}/start",
                body=maybe_transform({"wait": wait}, box_start_params.BoxStartParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, BoxStartResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def stop(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxStopResponse:
        """
        Stop box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxStopResponse,
            self._post(
                f"/boxes/{box_id}/stop",
                body=maybe_transform({"wait": wait}, box_stop_params.BoxStopParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, BoxStopResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def terminate(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Terminate box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/terminate",
            body=maybe_transform({"wait": wait}, box_terminate_params.BoxTerminateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def web_terminal_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxWebTerminalURLResponse:
        """
        Generate pre-signed web terminal url

        Args:
          expires_in: The web terminal will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 180m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/web-terminal-url",
            body=maybe_transform({"expires_in": expires_in}, box_web_terminal_url_params.BoxWebTerminalURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxWebTerminalURLResponse,
        )


class AsyncBoxesResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def fs(self) -> AsyncFsResource:
        return AsyncFsResource(self._client)

    @cached_property
    def browser(self) -> AsyncBrowserResource:
        return AsyncBrowserResource(self._client)

    @cached_property
    def android(self) -> AsyncAndroidResource:
        return AsyncAndroidResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBoxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncBoxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBoxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncBoxesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxRetrieveResponse:
        """
        Get box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxRetrieveResponse,
            await self._get(
                f"/boxes/{box_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, BoxRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        device_type: str | NotGiven = NOT_GIVEN,
        labels: object | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: List[Literal["all", "pending", "running", "error", "terminated"]] | NotGiven = NOT_GIVEN,
        type: List[Literal["all", "linux", "android"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxListResponse:
        """
        List box

        Args:
          device_type: Filter boxes by their device type (virtual, physical)

          labels: Filter boxes by their labels. Labels are key-value pairs that help identify and
              categorize boxes. Use this to filter boxes that match specific label criteria.
              For example, you can filter by project, environment, team, or any custom labels
              you've added to your boxes.

          page: Page number

          page_size: Page size

          status: Filter boxes by their current status (pending, running, stopped, error,
              terminated, all). Must be an array of statuses. Use 'all' to get boxes with any
              status.

          type: Filter boxes by their type (linux, android, all). Must be an array of types. Use
              'all' to get boxes of any type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/boxes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "device_type": device_type,
                        "labels": labels,
                        "page": page,
                        "page_size": page_size,
                        "status": status,
                        "type": type,
                    },
                    box_list_params.BoxListParams,
                ),
            ),
            cast_to=BoxListResponse,
        )

    async def create_android(
        self,
        *,
        config: box_create_android_params.Config | NotGiven = NOT_GIVEN,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidBox:
        """
        Create android box

        Args:
          config: Configuration for a Android box instance

          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/boxes/android",
            body=await async_maybe_transform(
                {
                    "config": config,
                    "wait": wait,
                },
                box_create_android_params.BoxCreateAndroidParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidBox,
        )

    async def create_linux(
        self,
        *,
        config: CreateBoxConfigParam | NotGiven = NOT_GIVEN,
        expires_in: str | NotGiven = NOT_GIVEN,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LinuxBox:
        """
        Create linux box

        Args:
          config: Configuration for a box instance

          expires_in: The box will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 60m

          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/boxes/linux",
            body=await async_maybe_transform(
                {
                    "config": config,
                    "expires_in": expires_in,
                    "wait": wait,
                },
                box_create_linux_params.BoxCreateLinuxParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LinuxBox,
        )

    async def execute_commands(
        self,
        box_id: str,
        *,
        commands: Union[str, List[str]],
        envs: object | NotGiven = NOT_GIVEN,
        api_timeout: str | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxExecuteCommandsResponse:
        """Exec command

        Args:
          commands: The command to run.

        Can be a single string or an array of strings

          envs: The environment variables to run the command

          api_timeout: The timeout of the command. If the command times out, the exit code will be 124.
              For example: 'timeout 5s sleep 10s' will result in exit code 124.

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30s

          working_dir: The working directory of the command. It not provided, the command will be run
              in the `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/commands",
            body=await async_maybe_transform(
                {
                    "commands": commands,
                    "envs": envs,
                    "api_timeout": api_timeout,
                    "working_dir": working_dir,
                },
                box_execute_commands_params.BoxExecuteCommandsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxExecuteCommandsResponse,
        )

    async def live_view_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxLiveViewURLResponse:
        """
        Generate pre-signed live view url

        Args:
          expires_in: The live view will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 180m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/live-view-url",
            body=await async_maybe_transform({"expires_in": expires_in}, box_live_view_url_params.BoxLiveViewURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxLiveViewURLResponse,
        )

    async def run_code(
        self,
        box_id: str,
        *,
        code: str,
        argv: List[str] | NotGiven = NOT_GIVEN,
        envs: object | NotGiven = NOT_GIVEN,
        language: Literal["bash", "python", "typescript"] | NotGiven = NOT_GIVEN,
        api_timeout: str | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxRunCodeResponse:
        """
        Run code on the box

        Args:
          code: The code to run

          argv: The arguments to run the code. For example, if you want to run "python index.py
              --help", you should pass ["--help"] as arguments.

          envs: The environment variables to run the code

          language: The language of the code.

          api_timeout: The timeout of the code execution. If the code execution times out, the exit
              code will be 124.

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 30s

          working_dir: The working directory of the code. It not provided, the code will be run in the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/run-code",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "argv": argv,
                    "envs": envs,
                    "language": language,
                    "api_timeout": api_timeout,
                    "working_dir": working_dir,
                },
                box_run_code_params.BoxRunCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxRunCodeResponse,
        )

    async def start(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxStartResponse:
        """
        Start box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxStartResponse,
            await self._post(
                f"/boxes/{box_id}/start",
                body=await async_maybe_transform({"wait": wait}, box_start_params.BoxStartParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, BoxStartResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def stop(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxStopResponse:
        """
        Stop box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            BoxStopResponse,
            await self._post(
                f"/boxes/{box_id}/stop",
                body=await async_maybe_transform({"wait": wait}, box_stop_params.BoxStopParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, BoxStopResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def terminate(
        self,
        box_id: str,
        *,
        wait: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Terminate box

        Args:
          wait: Wait for the box operation to be completed, default is true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/terminate",
            body=await async_maybe_transform({"wait": wait}, box_terminate_params.BoxTerminateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def web_terminal_url(
        self,
        box_id: str,
        *,
        expires_in: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BoxWebTerminalURLResponse:
        """
        Generate pre-signed web terminal url

        Args:
          expires_in: The web terminal will be alive for the given duration

              Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
              Example formats: "500ms", "30s", "5m", "1h" Default: 180m

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/web-terminal-url",
            body=await async_maybe_transform(
                {"expires_in": expires_in}, box_web_terminal_url_params.BoxWebTerminalURLParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BoxWebTerminalURLResponse,
        )


class BoxesResourceWithRawResponse:
    def __init__(self, boxes: BoxesResource) -> None:
        self._boxes = boxes

        self.retrieve = to_raw_response_wrapper(
            boxes.retrieve,
        )
        self.list = to_raw_response_wrapper(
            boxes.list,
        )
        self.create_android = to_raw_response_wrapper(
            boxes.create_android,
        )
        self.create_linux = to_raw_response_wrapper(
            boxes.create_linux,
        )
        self.execute_commands = to_raw_response_wrapper(
            boxes.execute_commands,
        )
        self.live_view_url = to_raw_response_wrapper(
            boxes.live_view_url,
        )
        self.run_code = to_raw_response_wrapper(
            boxes.run_code,
        )
        self.start = to_raw_response_wrapper(
            boxes.start,
        )
        self.stop = to_raw_response_wrapper(
            boxes.stop,
        )
        self.terminate = to_raw_response_wrapper(
            boxes.terminate,
        )
        self.web_terminal_url = to_raw_response_wrapper(
            boxes.web_terminal_url,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._boxes.actions)

    @cached_property
    def fs(self) -> FsResourceWithRawResponse:
        return FsResourceWithRawResponse(self._boxes.fs)

    @cached_property
    def browser(self) -> BrowserResourceWithRawResponse:
        return BrowserResourceWithRawResponse(self._boxes.browser)

    @cached_property
    def android(self) -> AndroidResourceWithRawResponse:
        return AndroidResourceWithRawResponse(self._boxes.android)


class AsyncBoxesResourceWithRawResponse:
    def __init__(self, boxes: AsyncBoxesResource) -> None:
        self._boxes = boxes

        self.retrieve = async_to_raw_response_wrapper(
            boxes.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            boxes.list,
        )
        self.create_android = async_to_raw_response_wrapper(
            boxes.create_android,
        )
        self.create_linux = async_to_raw_response_wrapper(
            boxes.create_linux,
        )
        self.execute_commands = async_to_raw_response_wrapper(
            boxes.execute_commands,
        )
        self.live_view_url = async_to_raw_response_wrapper(
            boxes.live_view_url,
        )
        self.run_code = async_to_raw_response_wrapper(
            boxes.run_code,
        )
        self.start = async_to_raw_response_wrapper(
            boxes.start,
        )
        self.stop = async_to_raw_response_wrapper(
            boxes.stop,
        )
        self.terminate = async_to_raw_response_wrapper(
            boxes.terminate,
        )
        self.web_terminal_url = async_to_raw_response_wrapper(
            boxes.web_terminal_url,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._boxes.actions)

    @cached_property
    def fs(self) -> AsyncFsResourceWithRawResponse:
        return AsyncFsResourceWithRawResponse(self._boxes.fs)

    @cached_property
    def browser(self) -> AsyncBrowserResourceWithRawResponse:
        return AsyncBrowserResourceWithRawResponse(self._boxes.browser)

    @cached_property
    def android(self) -> AsyncAndroidResourceWithRawResponse:
        return AsyncAndroidResourceWithRawResponse(self._boxes.android)


class BoxesResourceWithStreamingResponse:
    def __init__(self, boxes: BoxesResource) -> None:
        self._boxes = boxes

        self.retrieve = to_streamed_response_wrapper(
            boxes.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            boxes.list,
        )
        self.create_android = to_streamed_response_wrapper(
            boxes.create_android,
        )
        self.create_linux = to_streamed_response_wrapper(
            boxes.create_linux,
        )
        self.execute_commands = to_streamed_response_wrapper(
            boxes.execute_commands,
        )
        self.live_view_url = to_streamed_response_wrapper(
            boxes.live_view_url,
        )
        self.run_code = to_streamed_response_wrapper(
            boxes.run_code,
        )
        self.start = to_streamed_response_wrapper(
            boxes.start,
        )
        self.stop = to_streamed_response_wrapper(
            boxes.stop,
        )
        self.terminate = to_streamed_response_wrapper(
            boxes.terminate,
        )
        self.web_terminal_url = to_streamed_response_wrapper(
            boxes.web_terminal_url,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._boxes.actions)

    @cached_property
    def fs(self) -> FsResourceWithStreamingResponse:
        return FsResourceWithStreamingResponse(self._boxes.fs)

    @cached_property
    def browser(self) -> BrowserResourceWithStreamingResponse:
        return BrowserResourceWithStreamingResponse(self._boxes.browser)

    @cached_property
    def android(self) -> AndroidResourceWithStreamingResponse:
        return AndroidResourceWithStreamingResponse(self._boxes.android)


class AsyncBoxesResourceWithStreamingResponse:
    def __init__(self, boxes: AsyncBoxesResource) -> None:
        self._boxes = boxes

        self.retrieve = async_to_streamed_response_wrapper(
            boxes.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            boxes.list,
        )
        self.create_android = async_to_streamed_response_wrapper(
            boxes.create_android,
        )
        self.create_linux = async_to_streamed_response_wrapper(
            boxes.create_linux,
        )
        self.execute_commands = async_to_streamed_response_wrapper(
            boxes.execute_commands,
        )
        self.live_view_url = async_to_streamed_response_wrapper(
            boxes.live_view_url,
        )
        self.run_code = async_to_streamed_response_wrapper(
            boxes.run_code,
        )
        self.start = async_to_streamed_response_wrapper(
            boxes.start,
        )
        self.stop = async_to_streamed_response_wrapper(
            boxes.stop,
        )
        self.terminate = async_to_streamed_response_wrapper(
            boxes.terminate,
        )
        self.web_terminal_url = async_to_streamed_response_wrapper(
            boxes.web_terminal_url,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._boxes.actions)

    @cached_property
    def fs(self) -> AsyncFsResourceWithStreamingResponse:
        return AsyncFsResourceWithStreamingResponse(self._boxes.fs)

    @cached_property
    def browser(self) -> AsyncBrowserResourceWithStreamingResponse:
        return AsyncBrowserResourceWithStreamingResponse(self._boxes.browser)

    @cached_property
    def android(self) -> AsyncAndroidResourceWithStreamingResponse:
        return AsyncAndroidResourceWithStreamingResponse(self._boxes.android)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BoxRunCodeParams"]


class BoxRunCodeParams(TypedDict, total=False):
    code: Required[str]
    """The code to run"""

    argv: List[str]
    """The arguments to run the code.

    For example, if you want to run "python index.py --help", you should pass
    ["--help"] as arguments.
    """

    envs: object
    """The environment variables to run the code"""

    language: Literal["bash", "python", "typescript"]
    """The language of the code."""

    api_timeout: Annotated[str, PropertyInfo(alias="timeout")]
    """The timeout of the code execution.

    If the code execution times out, the exit code will be 124.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 30s
    """

    working_dir: Annotated[str, PropertyInfo(alias="workingDir")]
    """The working directory of the code.

    It not provided, the code will be run in the `box.config.workingDir` directory.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = [
    "ActionScrollResponse",
    "ActionIncludeScreenshotResult",
    "ActionIncludeScreenshotResultScreenshot",
    "ActionIncludeScreenshotResultScreenshotAfter",
    "ActionIncludeScreenshotResultScreenshotBefore",
    "ActionIncludeScreenshotResultScreenshotTrace",
    "ActionCommonResult",
]


class ActionIncludeScreenshotResultScreenshotAfter(BaseModel):
    uri: str
    """URI of the screenshot after the action"""


class ActionIncludeScreenshotResultScreenshotBefore(BaseModel):
    uri: str
    """URI of the screenshot before the action"""


class ActionIncludeScreenshotResultScreenshotTrace(BaseModel):
    uri: str
    """URI of the screenshot with operation trace"""


class ActionIncludeScreenshotResultScreenshot(BaseModel):
    after: ActionIncludeScreenshotResultScreenshotAfter
    """Screenshot taken after action execution"""

    before: ActionIncludeScreenshotResultScreenshotBefore
    """Screenshot taken before action execution"""

    trace: ActionIncludeScreenshotResultScreenshotTrace
    """Screenshot with action operation trace"""


class ActionIncludeScreenshotResult(BaseModel):
    screenshot: ActionIncludeScreenshotResultScreenshot
    """Complete screenshot result with operation trace, before and after images"""


class ActionCommonResult(BaseModel):
    message: str
    """message"""


ActionScrollResponse: TypeAlias = Union[ActionIncludeScreenshotResult, ActionCommonResult]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["FWriteResponse"]


class FWriteResponse(BaseModel):
    last_modified: datetime = FieldInfo(alias="lastModified")
    """Last modified time of the file"""

    mode: str
    """File metadata"""

    name: str
    """Name of the file"""

    path: str
    """Full path to the file in the box"""

    size: str
    """Size of the file"""

    type: Literal["file"]
    """File type indicator"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["AndroidGetConnectAddressResponse"]


class AndroidGetConnectAddressResponse(BaseModel):
    adb: str
    """Android adb connect address.

    use `adb connect <adbConnectAddress>` to connect to the Android device
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ActionScrollParams"]


class ActionScrollParams(TypedDict, total=False):
    scroll_x: Required[Annotated[float, PropertyInfo(alias="scrollX")]]
    """Horizontal scroll amount"""

    scroll_y: Required[Annotated[float, PropertyInfo(alias="scrollY")]]
    """Vertical scroll amount"""

    x: Required[float]
    """X coordinate of the scroll position"""

    y: Required[float]
    """Y coordinate of the scroll position"""

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """Whether to include screenshots in the action response.

    If false, the screenshot object will still be returned but with empty URIs.
    Default is false.
    """

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """Type of the URI. default is base64."""

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["FInfoResponse", "File", "Dir"]


class File(BaseModel):
    last_modified: datetime = FieldInfo(alias="lastModified")
    """Last modified time of the file"""

    mode: str
    """File metadata"""

    name: str
    """Name of the file"""

    path: str
    """Full path to the file in the box"""

    size: str
    """Size of the file"""

    type: Literal["file"]
    """File type indicator"""


class Dir(BaseModel):
    last_modified: datetime = FieldInfo(alias="lastModified")
    """Last modified time of the directory"""

    mode: str
    """Directory metadata"""

    name: str
    """Name of the directory"""

    path: str
    """Full path to the directory in the box"""

    type: Literal["dir"]
    """Directory type indicator"""


FInfoResponse: TypeAlias = Union[File, Dir]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ActionScreenshotResponse"]


class ActionScreenshotResponse(BaseModel):
    uri: str
    """URL of the screenshot"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BoxRunCodeResponse"]


class BoxRunCodeResponse(BaseModel):
    exit_code: float = FieldInfo(alias="exitCode")
    """The exit code of the code"""

    stderr: str
    """The stderr of the code"""

    stdout: str
    """The stdout of the code"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .android_app import AndroidApp

__all__ = ["AndroidListAppResponse"]


class AndroidListAppResponse(BaseModel):
    data: List[AndroidApp]
    """App list"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven, FileTypes
from ...._utils import extract_files, required_args, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import (
    android_open_params,
    android_install_params,
    android_restart_params,
    android_restore_params,
    android_list_pkg_params,
    android_uninstall_params,
    android_list_pkg_simple_params,
)
from ....types.v1.boxes.android_app import AndroidApp
from ....types.v1.boxes.android_get_response import AndroidGetResponse
from ....types.v1.boxes.android_install_response import AndroidInstallResponse
from ....types.v1.boxes.android_list_app_response import AndroidListAppResponse
from ....types.v1.boxes.android_list_pkg_response import AndroidListPkgResponse
from ....types.v1.boxes.android_list_activities_response import AndroidListActivitiesResponse
from ....types.v1.boxes.android_list_pkg_simple_response import AndroidListPkgSimpleResponse
from ....types.v1.boxes.android_get_connect_address_response import AndroidGetConnectAddressResponse

__all__ = ["AndroidResource", "AsyncAndroidResource"]


class AndroidResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AndroidResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AndroidResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AndroidResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AndroidResourceWithStreamingResponse(self)

    def backup(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Backup

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/backup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def backup_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Backup all

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/backup-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def close(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/close",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def close_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close all apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/close-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetResponse:
        """
        Get pkg

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetResponse,
        )

    def get_app(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidApp:
        """
        Get app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/apps/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidApp,
        )

    def get_connect_address(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetConnectAddressResponse:
        """
        Get connect address

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/connect-address",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetConnectAddressResponse,
        )

    @overload
    def install(
        self,
        box_id: str,
        *,
        apk: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: APK file to install (max file size: 512MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def install(
        self,
        box_id: str,
        *,
        apk: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: HTTP URL to download APK file (max file size: 512MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["apk"])
    def install(
        self,
        box_id: str,
        *,
        apk: FileTypes | str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal({"apk": apk})
        files = extract_files(cast(Mapping[str, object], body), paths=[["apk"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages",
            body=maybe_transform(body, android_install_params.AndroidInstallParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidInstallResponse,
        )

    def list_activities(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListActivitiesResponse:
        """
        Get pkg activities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/{package_name}/activities",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListActivitiesResponse,
        )

    def list_app(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListAppResponse:
        """
        List apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/apps",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListAppResponse,
        )

    def list_pkg(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        running_filter: List[Literal["running", "notRunning"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgResponse:
        """Retrieve detailed information for all installed pkg.

        This endpoint provides
        comprehensive pkg details

        Args:
          pkg_type: Package type: system or thirdParty, default is thirdParty

          running_filter: Filter pkgs by running status: running (show only running pkgs), notRunning
              (show only non-running pkgs). Default is all

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "pkg_type": pkg_type,
                        "running_filter": running_filter,
                    },
                    android_list_pkg_params.AndroidListPkgParams,
                ),
            ),
            cast_to=AndroidListPkgResponse,
        )

    def list_pkg_simple(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgSimpleResponse:
        """A faster endpoint to quickly retrieve basic pkg information.

        This API provides
        better performance for scenarios where you need to get essential pkg details
        quickly

        Args:
          pkg_type: Package type: system or thirdParty, default is thirdParty

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/simple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"pkg_type": pkg_type}, android_list_pkg_simple_params.AndroidListPkgSimpleParams
                ),
            ),
            cast_to=AndroidListPkgSimpleResponse,
        )

    def open(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Open app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/open",
            body=maybe_transform({"activity_name": activity_name}, android_open_params.AndroidOpenParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def restart(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restart app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/restart",
            body=maybe_transform({"activity_name": activity_name}, android_restart_params.AndroidRestartParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def restore(
        self,
        box_id: str,
        *,
        backup: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restore

        Args:
          backup: Backup file to restore (max file size: 100MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/restore",
            body=maybe_transform({"backup": backup}, android_restore_params.AndroidRestoreParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def uninstall(
        self,
        package_name: str,
        *,
        box_id: str,
        keep_data: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Uninstall app

        Args:
          keep_data: uninstalls the pkg while retaining the data/cache

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/boxes/{box_id}/android/packages/{package_name}",
            body=maybe_transform({"keep_data": keep_data}, android_uninstall_params.AndroidUninstallParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAndroidResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAndroidResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAndroidResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAndroidResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncAndroidResourceWithStreamingResponse(self)

    async def backup(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Backup

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/backup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def backup_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Backup all

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/backup-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def close(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/close",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def close_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close all apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/close-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetResponse:
        """
        Get pkg

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetResponse,
        )

    async def get_app(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidApp:
        """
        Get app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/apps/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidApp,
        )

    async def get_connect_address(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetConnectAddressResponse:
        """
        Get connect address

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/connect-address",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetConnectAddressResponse,
        )

    @overload
    async def install(
        self,
        box_id: str,
        *,
        apk: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: APK file to install (max file size: 512MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def install(
        self,
        box_id: str,
        *,
        apk: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: HTTP URL to download APK file (max file size: 512MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["apk"])
    async def install(
        self,
        box_id: str,
        *,
        apk: FileTypes | str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal({"apk": apk})
        files = extract_files(cast(Mapping[str, object], body), paths=[["apk"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages",
            body=await async_maybe_transform(body, android_install_params.AndroidInstallParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidInstallResponse,
        )

    async def list_activities(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListActivitiesResponse:
        """
        Get pkg activities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/{package_name}/activities",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListActivitiesResponse,
        )

    async def list_app(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListAppResponse:
        """
        List apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/apps",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListAppResponse,
        )

    async def list_pkg(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        running_filter: List[Literal["running", "notRunning"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgResponse:
        """Retrieve detailed information for all installed pkg.

        This endpoint provides
        comprehensive pkg details

        Args:
          pkg_type: Package type: system or thirdParty, default is thirdParty

          running_filter: Filter pkgs by running status: running (show only running pkgs), notRunning
              (show only non-running pkgs). Default is all

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "pkg_type": pkg_type,
                        "running_filter": running_filter,
                    },
                    android_list_pkg_params.AndroidListPkgParams,
                ),
            ),
            cast_to=AndroidListPkgResponse,
        )

    async def list_pkg_simple(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgSimpleResponse:
        """A faster endpoint to quickly retrieve basic pkg information.

        This API provides
        better performance for scenarios where you need to get essential pkg details
        quickly

        Args:
          pkg_type: Package type: system or thirdParty, default is thirdParty

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/simple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"pkg_type": pkg_type}, android_list_pkg_simple_params.AndroidListPkgSimpleParams
                ),
            ),
            cast_to=AndroidListPkgSimpleResponse,
        )

    async def open(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Open app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/open",
            body=await async_maybe_transform({"activity_name": activity_name}, android_open_params.AndroidOpenParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def restart(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restart app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/restart",
            body=await async_maybe_transform(
                {"activity_name": activity_name}, android_restart_params.AndroidRestartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def restore(
        self,
        box_id: str,
        *,
        backup: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restore

        Args:
          backup: Backup file to restore (max file size: 100MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/restore",
            body=await async_maybe_transform({"backup": backup}, android_restore_params.AndroidRestoreParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def uninstall(
        self,
        package_name: str,
        *,
        box_id: str,
        keep_data: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Uninstall app

        Args:
          keep_data: uninstalls the pkg while retaining the data/cache

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/boxes/{box_id}/android/packages/{package_name}",
            body=await async_maybe_transform({"keep_data": keep_data}, android_uninstall_params.AndroidUninstallParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AndroidResourceWithRawResponse:
    def __init__(self, android: AndroidResource) -> None:
        self._android = android

        self.backup = to_custom_raw_response_wrapper(
            android.backup,
            BinaryAPIResponse,
        )
        self.backup_all = to_custom_raw_response_wrapper(
            android.backup_all,
            BinaryAPIResponse,
        )
        self.close = to_raw_response_wrapper(
            android.close,
        )
        self.close_all = to_raw_response_wrapper(
            android.close_all,
        )
        self.get = to_raw_response_wrapper(
            android.get,
        )
        self.get_app = to_raw_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = to_raw_response_wrapper(
            android.get_connect_address,
        )
        self.install = to_raw_response_wrapper(
            android.install,
        )
        self.list_activities = to_raw_response_wrapper(
            android.list_activities,
        )
        self.list_app = to_raw_response_wrapper(
            android.list_app,
        )
        self.list_pkg = to_raw_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = to_raw_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = to_raw_response_wrapper(
            android.open,
        )
        self.restart = to_raw_response_wrapper(
            android.restart,
        )
        self.restore = to_raw_response_wrapper(
            android.restore,
        )
        self.uninstall = to_raw_response_wrapper(
            android.uninstall,
        )


class AsyncAndroidResourceWithRawResponse:
    def __init__(self, android: AsyncAndroidResource) -> None:
        self._android = android

        self.backup = async_to_custom_raw_response_wrapper(
            android.backup,
            AsyncBinaryAPIResponse,
        )
        self.backup_all = async_to_custom_raw_response_wrapper(
            android.backup_all,
            AsyncBinaryAPIResponse,
        )
        self.close = async_to_raw_response_wrapper(
            android.close,
        )
        self.close_all = async_to_raw_response_wrapper(
            android.close_all,
        )
        self.get = async_to_raw_response_wrapper(
            android.get,
        )
        self.get_app = async_to_raw_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = async_to_raw_response_wrapper(
            android.get_connect_address,
        )
        self.install = async_to_raw_response_wrapper(
            android.install,
        )
        self.list_activities = async_to_raw_response_wrapper(
            android.list_activities,
        )
        self.list_app = async_to_raw_response_wrapper(
            android.list_app,
        )
        self.list_pkg = async_to_raw_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = async_to_raw_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = async_to_raw_response_wrapper(
            android.open,
        )
        self.restart = async_to_raw_response_wrapper(
            android.restart,
        )
        self.restore = async_to_raw_response_wrapper(
            android.restore,
        )
        self.uninstall = async_to_raw_response_wrapper(
            android.uninstall,
        )


class AndroidResourceWithStreamingResponse:
    def __init__(self, android: AndroidResource) -> None:
        self._android = android

        self.backup = to_custom_streamed_response_wrapper(
            android.backup,
            StreamedBinaryAPIResponse,
        )
        self.backup_all = to_custom_streamed_response_wrapper(
            android.backup_all,
            StreamedBinaryAPIResponse,
        )
        self.close = to_streamed_response_wrapper(
            android.close,
        )
        self.close_all = to_streamed_response_wrapper(
            android.close_all,
        )
        self.get = to_streamed_response_wrapper(
            android.get,
        )
        self.get_app = to_streamed_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = to_streamed_response_wrapper(
            android.get_connect_address,
        )
        self.install = to_streamed_response_wrapper(
            android.install,
        )
        self.list_activities = to_streamed_response_wrapper(
            android.list_activities,
        )
        self.list_app = to_streamed_response_wrapper(
            android.list_app,
        )
        self.list_pkg = to_streamed_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = to_streamed_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = to_streamed_response_wrapper(
            android.open,
        )
        self.restart = to_streamed_response_wrapper(
            android.restart,
        )
        self.restore = to_streamed_response_wrapper(
            android.restore,
        )
        self.uninstall = to_streamed_response_wrapper(
            android.uninstall,
        )


class AsyncAndroidResourceWithStreamingResponse:
    def __init__(self, android: AsyncAndroidResource) -> None:
        self._android = android

        self.backup = async_to_custom_streamed_response_wrapper(
            android.backup,
            AsyncStreamedBinaryAPIResponse,
        )
        self.backup_all = async_to_custom_streamed_response_wrapper(
            android.backup_all,
            AsyncStreamedBinaryAPIResponse,
        )
        self.close = async_to_streamed_response_wrapper(
            android.close,
        )
        self.close_all = async_to_streamed_response_wrapper(
            android.close_all,
        )
        self.get = async_to_streamed_response_wrapper(
            android.get,
        )
        self.get_app = async_to_streamed_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = async_to_streamed_response_wrapper(
            android.get_connect_address,
        )
        self.install = async_to_streamed_response_wrapper(
            android.install,
        )
        self.list_activities = async_to_streamed_response_wrapper(
            android.list_activities,
        )
        self.list_app = async_to_streamed_response_wrapper(
            android.list_app,
        )
        self.list_pkg = async_to_streamed_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = async_to_streamed_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = async_to_streamed_response_wrapper(
            android.open,
        )
        self.restart = async_to_streamed_response_wrapper(
            android.restart,
        )
        self.restore = async_to_streamed_response_wrapper(
            android.restore,
        )
        self.uninstall = async_to_streamed_response_wrapper(
            android.uninstall,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ActionSwipeParams", "SwipeSimple", "SwipeAdvanced", "SwipeAdvancedEnd", "SwipeAdvancedStart"]


class SwipeSimple(TypedDict, total=False):
    direction: Required[Literal["up", "down", "left", "right", "upLeft", "upRight", "downLeft", "downRight"]]
    """Direction to swipe.

    The gesture will be performed from the center of the screen towards this
    direction.
    """

    distance: float
    """Distance of the swipe in pixels.

    If not provided, the swipe will be performed from the center of the screen to
    the screen edge
    """

    duration: str
    """Duration of the swipe

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms
    """

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """Whether to include screenshots in the action response.

    If false, the screenshot object will still be returned but with empty URIs.
    Default is false.
    """

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """Type of the URI. default is base64."""

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """


class SwipeAdvanced(TypedDict, total=False):
    end: Required[SwipeAdvancedEnd]
    """Swipe path"""

    start: Required[SwipeAdvancedStart]
    """Swipe path"""

    duration: str
    """Duration of the swipe

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms
    """

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """Whether to include screenshots in the action response.

    If false, the screenshot object will still be returned but with empty URIs.
    Default is false.
    """

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """Type of the URI. default is base64."""

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """


class SwipeAdvancedEnd(TypedDict, total=False):
    x: Required[float]
    """Start/end x coordinate of the swipe path"""

    y: Required[float]
    """Start/end y coordinate of the swipe path"""


class SwipeAdvancedStart(TypedDict, total=False):
    x: Required[float]
    """Start/end x coordinate of the swipe path"""

    y: Required[float]
    """Start/end y coordinate of the swipe path"""


ActionSwipeParams: TypeAlias = Union[SwipeSimple, SwipeAdvanced]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["BoxWebTerminalURLResponse"]


class BoxWebTerminalURLResponse(BaseModel):
    url: str
    """Web terminal url"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .linux_box import LinuxBox as LinuxBox
from .android_box import AndroidBox as AndroidBox
from .box_list_params import BoxListParams as BoxListParams
from .box_stop_params import BoxStopParams as BoxStopParams
from .box_start_params import BoxStartParams as BoxStartParams
from .box_list_response import BoxListResponse as BoxListResponse
from .box_stop_response import BoxStopResponse as BoxStopResponse
from .box_start_response import BoxStartResponse as BoxStartResponse
from .box_run_code_params import BoxRunCodeParams as BoxRunCodeParams
from .box_terminate_params import BoxTerminateParams as BoxTerminateParams
from .box_retrieve_response import BoxRetrieveResponse as BoxRetrieveResponse
from .box_run_code_response import BoxRunCodeResponse as BoxRunCodeResponse
from .box_create_linux_params import BoxCreateLinuxParams as BoxCreateLinuxParams
from .create_box_config_param import CreateBoxConfigParam as CreateBoxConfigParam
from .box_live_view_url_params import BoxLiveViewURLParams as BoxLiveViewURLParams
from .box_create_android_params import BoxCreateAndroidParams as BoxCreateAndroidParams
from .box_live_view_url_response import BoxLiveViewURLResponse as BoxLiveViewURLResponse
from .box_execute_commands_params import BoxExecuteCommandsParams as BoxExecuteCommandsParams
from .box_web_terminal_url_params import BoxWebTerminalURLParams as BoxWebTerminalURLParams
from .box_execute_commands_response import BoxExecuteCommandsResponse as BoxExecuteCommandsResponse
from .box_web_terminal_url_response import BoxWebTerminalURLResponse as BoxWebTerminalURLResponse
